@@ -281,6 +281,10 @@ typedef int64_t		INT64;
 
 #if defined(__KERNEL__)
 
+#if ((LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,19)))
+#define URB_ONLY_CALLBACK
+#endif
+
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,19))
 #define PT_REGS    , struct pt_regs *regs
 #else
@@ -490,6 +494,12 @@ create_proc_read_entry(const char *name, mode_t mode,
         } while (0)
 #endif
 
+#ifndef container_of
+#define container_of(ptr, type, member) ({			\
+        const typeof( ((type *)0)->member ) *__mptr = (ptr);	\
+        (type *)( (char *)__mptr - offsetof(type,member) );})
+#endif
+
 #ifndef INIT_WORK
 #define work_struct tq_struct
 
@@ -500,10 +510,18 @@ create_proc_read_entry(const char *name, mode_t mode,
 #endif
 
 #define flush_scheduled_work  flush_scheduled_tasks
-#define INIT_WORK(_wq, _routine, _data)  INIT_TQUEUE(_wq, _routine, _data)
-#define PREPARE_WORK(_wq, _routine, _data)  PREPARE_TQUEUE(_wq, _routine, _data)
+#define INIT_WORK2(_wq, _routine)  INIT_TQUEUE(_wq, (void (*)(void *))_routine, _wq)
 #endif
-#endif // < 2.5 kernel
+
+#else // >= 2.5 kernel
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,20)
+#define INIT_WORK2(_wq, _routine)	INIT_WORK(_wq, (void (*)(void *))_routine, _wq)
+#else
+#define INIT_WORK2(_wq, _routine)	INIT_WORK(_wq, _routine)
+#endif
+
+#endif // >= 2.5 kernel
 
 #if (LINUX_VERSION_CODE < KERNEL_VERSION(2,3,38))
 typedef struct device netdevice_t;
